<?php

namespace Drupal\private_media;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Database\Driver\mysql\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class PrivateMediaService.
 */
class PrivateMediaService {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Database\Driver\mysql\Connection definition.
   *
   * @var \Drupal\Core\Database\Driver\mysql\Connection
   */
  protected $database;

  const FILE_UNDETERMINED = 0;
  const FILE_PRIVATE = 1;
  const FILE_PUBLIC = 2;
  const FILE_MODULE_UNINSTALL = 999;

  const FILE_SYSTEMS = [
    self::FILE_UNDETERMINED => 'private',
    self::FILE_PRIVATE => 'private',
    self::FILE_PUBLIC => 'public',
    self::FILE_MODULE_UNINSTALL => 'public',
  ];

  /**
   * Constructs a new PrivateMediaService object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Connection $database, FileSystemInterface $fs) {
    $this->entityTypeManager = $entity_type_manager;
    $this->database = $database;
    $this->fileSystem = $fs;
  }

  /**
   * Responds to an entity operation.
   *
   * @param Drupal\Core\Entity\EntityInterface $entity
   *   The entity being operated on.
   * @param string $op
   *   The operation name.
   */
  public function mediaEntityOperation(EntityInterface $entity, string $op) {
    if (method_exists($entity, 'hasField') && $entity->hasField('moderation_state')) {
      $field_names = ['thumbnail' => 'thumbnail'];

      $res = $this->database->select('key_value', 'kv')
        ->condition('kv.collection', 'entity.definitions.bundle_field_map')
        ->condition('kv.name', 'media')
        ->fields('kv')
        ->execute();
      foreach ($res as $row) {
        $data = unserialize($row->value);
        foreach (unserialize($row->value) as $field_name => $field_info) {
          if (in_array($field_info['type'], ['image', 'file'])) {
            $field_names[$field_name] = $field_name;
          }
        }
      }

      // Load all of the related revisions.
      $ms = $this->entityTypeManager->getStorage('media');
      $rids = $this->database->select('media_revision', 'rev')
        ->condition('rev.mid', $entity->id())
        ->fields('rev', ['vid'])
        ->orderBy('vid', 'DESC')
        ->execute()->fetchCol();

      $references = [];
      $found_latest = FALSE;
      foreach ($rids as $rid) {
        list($revision, $is_current) = ($rid == $entity->vid->value) ? [$entity, TRUE] : [$ms->loadRevision($rid), FALSE];
        // Determine this flag for the entire revision, not just the first file.
        $find_public = !$found_latest;
        foreach ($field_names as $field_name) {
          if ($revision->hasField($field_name)) {
            foreach ($revision->get($field_name)->referencedEntities() as $file_entity) {
              $references[$file_entity->id()] = $references[$file_entity->id()] ?? [
                'entity' => $file_entity,
                'system' => self::FILE_UNDETERMINED,
              ];

              if ($op === 'uninstall') {
                $references[$file_entity->id()]['system'] = self::FILE_MODULE_UNINSTALL;
              }
              elseif (!$references[$file_entity->id()]['system'] && $find_public) {
                $mod_state = $revision->get('moderation_state')->value;
                switch ($mod_state) {
                  case 'published':
                    $references[$file_entity->id()]['system'] = self::FILE_PUBLIC;
                    $found_latest = TRUE;
                    break;

                  case 'archived':
                  case 'recycled':
                    $references[$file_entity->id()]['system'] = self::FILE_PRIVATE;
                    $found_latest = TRUE;
                    break;
                }
              }
            }
          }
        }
      }

      foreach ($references as $fid => $reference) {
        $path = explode('://', $reference['entity']->getFileUri(), 2);
        if (count($path) === 2) {
          $newuri = self::FILE_SYSTEMS[$reference['system']] . '://' . $path[1];
          $this->moveFile($reference['entity'], $newuri);
        }
      }
    }
  }

  /**
   * Move the file from source uri to target uri.
   */
  private function moveFile(EntityInterface $fileEntity, string $newuri) {
    if (empty($fileEntity) || !method_exists($fileEntity, 'getFileUri') || !method_exists($fileEntity, 'setFileUri')) {
      return FALSE;
    }

    $absolduri = $this->fileSystem->realpath($fileEntity->getFileUri());
    $absnewuri = $this->fileSystem->realpath($newuri);

    if ($absolduri === $absnewuri) {
      return TRUE;
    }

    $oldexists = file_exists($absolduri);
    $newexists = file_exists($absnewuri);

    if ($oldexists && !$newexists) {
      $newdir = pathinfo($absnewuri, PATHINFO_DIRNAME);
      if ($this->fileSystem->prepareDirectory($newdir, FileSystemInterface::CREATE_DIRECTORY)) {
        // Move file and update uri.
        if (rename($absolduri, $absnewuri)) {
          $fileEntity->setFileUri($newuri);
          $fileEntity->save();
          return TRUE;
        }
      }
    }

    if (!$oldexists && $newexists) {
      // File already there, update URI only.
      $fileEntity->setFileUri($absnewuri);
      $fileEntity->save();
      return TRUE;
    }

    return FALSE;
  }

  /**
   * Wrapped by hook_help().
   *
   * @param mixed $route_name
   *   The route name.
   * @param Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match interface.
   */
  public function privateMediaHelp($route_name, RouteMatchInterface $route_match) {
    switch ($route_name) {
      // Main module help for the private_media module.
      case 'help.page.private_media':
        $output = '';
        $output .= '<h3>' . t('About') . '</h3>';
        $output .= '<p>' . t('Moves media files to the private file system when not referenced by a published revision') . '</p>';
        return $output;

      default:
    }
  }

}
